import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.*;

/**
 * contains all the variables used by the enemies
 * every enemy has health, attack, and destroy attributes
 * 
 * @author Team Link
 * @version 5/10/2012
 */
public class Enemies  extends Actor
{

    protected enum State {MOVE, KNOCKBACK, CONTACT};

    /**
     * Images used for animation
     */
    protected GreenfootImage[] images;
    
    LinkWorld lw = (LinkWorld) getWorld();
    
    /**
     * The worlds reference
     */
    protected LinkWorld theWorld;
    
    protected State state;
    
   
    private int health = 3;   
    
    /**
     * The enemies x and y positions
     */
    protected int xPose;
    protected int yPose;
    
    protected boolean alive;
    protected boolean hurt;
    public static int kills=0;
    protected static final int FINAL_DELAY = 5;
    protected static final int maxHurtTime = 40;
    protected int hurtDelay = FINAL_DELAY;
    protected int hurtTimer = maxHurtTime;
    protected boolean isHurt = false;
    protected boolean inKnockback = false;
    protected boolean takeHealth=true;
    
    public int direction=0;
    int imgRot = 0;
    
    /**
     * Returns the enemies x position
     * 
     * @return The x position
     */
    public int xPose()
    {
        return xPose;
    }

    /**
     * Returns the enemies y position
     * 
     * @return The y position
     */
    public int yPose()
    {
        return yPose;
    }
        
    /**
     * Decreases this enemy's health equal to amount.
     * 
     * @param amount The amount of health subtracted from this enemy
     */
    public void decreaseHealth(int amount)
    {
        health -= amount;
        if(health <= 0){
            destroy(); kills++; UI.kill.play();
            if(kills>5){ UI.miniBoss=1;}
        }
            
    }
    
    /**
     * Called to remove the current enemy
     */
    private void removeSelf()
    {
        this.getWorld().removeObject(this);
    }
    
    /**
     * Sets alive boolean to false and removes enemy from the world.
     */
    protected void destroy()
    {
        alive = false;
        removeSelf();
    }
    
    /**
     * Checks if enemy is alive.
     */
    protected boolean isAlive()
    {
        return this.alive;
    }
    
    /**
     * Reverts an enemy to their initialization state. 
     */
    public void resetEnemy(){
        this.alive=true;
        this.inKnockback=false;
        this.isHurt = false;
        this.takeHealth = true;
        this.health = 3;
        this.hurtTimer = maxHurtTime;
    } 
    
    /**
     * knockback(int dir)
     * @param dir current direction enemy is facing
     * reset variables to start animation when enemy is hit
     */
    protected void knockBack( int dir){
        //if the octorock has not finished the knockback cycle just return
        if(hurtTimer < maxHurtTime){return;}
        //System.out.println("knockback");
        this.direction = dir;
        this.hurtTimer = 0;
        this.hurtDelay = 0;
        this.inKnockback = true;
    }
    
    /**
     * isHit() when enemy intersects with Link
     */
    public void isHit()
    {
        Actor link = getOneIntersectingObject(Link.class);
        
        if(link != null)
        {
            if(Link.isAttacking == true){
                this.decreaseHealth(1);
                this.hurt=true;
                UI.hit.play();
                this.takeHealth=false;
                this.knockBack(direction);
            }else if(Link.isAttacking == false){
                Link.linkIsHit();
            }
            
            if(state != null)
                state = State.CONTACT;
            else{}
                //removeSelf();
        }            
    }   

}
