import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.*;
/**
 * This class is for the Link actor, the main character of the game, and the only character
 * directly controlled by the user. Allows for movement and attacking.
 * 
 * @author Matthew Kurupas
 * @version February 23, 2012
 */
public class Link  extends Actor
{

    //Enum of the directions up, down, left, and right, respectively
    private enum Direction {UP, DOWN, LEFT, RIGHT}
    
    //Allows for easy change of Link's movement speed
    private int fairySecret=0;
    private int secret=0;
    private int moveSpeed = 2;
    public static int health = 5;
    private static int recovery=0;
    //Initialize image variables
    private GreenfootImage LinkDown0 = new GreenfootImage("linkDown0.png");
    private GreenfootImage LinkDown1 = new GreenfootImage("linkDown1.png");
    
    private GreenfootImage LinkSwordDown = new GreenfootImage("linkSwordDown.png");
    private GreenfootImage LinkSwordUp = new GreenfootImage("linkSwordUp.png");
    private GreenfootImage LinkSwordLeft = new GreenfootImage("linkSwordLeft.png");
    private GreenfootImage LinkSwordRight = new GreenfootImage("linkSwordRight.png");
    
    //Initializes variables used for animations
    private static final int FINAL_DELAY = 15;
    private static boolean isImage1 = false;
    private static int delay = FINAL_DELAY;
    private static int attackTime = 10;
    private static int distance = 30;
    private static int timer = 0;
    private static int counter = 0;
    private static final int shift = 30;
    private static double temp = 0.0;
    private static int newScreenPos = 0;
    private static int oldScreenPos = 0;
    private static Direction currentDirection = Direction.DOWN;
    private static Direction attackDirection = Direction.DOWN;
    private static Direction knockbackDirection = Direction.DOWN;
    private static int spin = 4;
    
    public static boolean isAttacking = false;
    public static boolean isGameOver = false;
    public static boolean isKnockback = false;
    public static GameOverScreen gos = new GameOverScreen();
    public static EndScreen endScreen = new EndScreen();
    
    /**
     * Act - do whatever the Link wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        //Checks the recovery of Link
        if(recovery>0){
            recovery--;
            }
        
        //When Link is hit, Link will smoothly be knocked back
        if(isKnockback == true && counter <= 5){
            
            move(knockbackDirection, 10);
            counter++;
            if(counter == 4){
                isKnockback = false;
            }
        }else{
        
        //Causes Link to attack for more than half of a second when space
        //bar is hit.
        if(isAttacking == true && timer % 15 == 0){
            stopAttacking(attackDirection);
            isAttacking = false;
        }
        
        //Checks if the game is done spawning enemies, and ends game if true
        if(Spawner.finish == true){
            UI.endGame.play();
            //move(Direction.DOWN, 500);
            this.getImage().setTransparency(0);
            getWorld().addObject(endScreen, 126, 110);            
            Greenfoot.stop();
        }
        
        //Checks to see if the game is over (Link ran out of hearts). If true,
        //it starts new music, does Link's death animation, and shows end screen.
        if(isGameOver == true){
            UI.music.stop();
            UI.gameOverSFX.play();
            setImage("linkDown0.png");
            Greenfoot.delay(spin);
            setImage("linkLeft0.png");
            Greenfoot.delay(spin);
            setImage("linkUp0.png");
            Greenfoot.delay(spin);
            setImage("linkRight0.png");
            Greenfoot.delay(spin);
            setImage("linkDown0.png");
            Greenfoot.delay(spin);
            setImage("linkLeft0.png");
            Greenfoot.delay(spin);
            setImage("linkUp0.png");
            Greenfoot.delay(spin);
            setImage("linkRight0.png");
            Greenfoot.delay(spin);
            setImage("linkDown0.png");
            Greenfoot.delay(spin);
            setImage("linkLeft0.png");
            Greenfoot.delay(spin);
            setImage("linkUp0.png");
            Greenfoot.delay(spin);
            setImage("linkRight0.png");
            Greenfoot.delay(spin);
            setImage("linkDown0.png");
            setImage("deadLink.png");
            //move(Direction.DOWN, 500);
            this.getImage().setTransparency(0);
            getWorld().addObject(gos, 126, 110);
            
            Greenfoot.stop();
        }
        
        //Checks if the player is attack. If he is not, allows the user to control
        //Link.
        if(isAttacking == false){
            playerInput();
        }
        
        timer++;
        counter = 0;
        }
    }  
    
    /**
     * Looks for input from player to manipulate Link. Can be moved up, down, left,
     * and right by using the respective arrow keys. Can also attack via space bar.
     * 
     * This method also contains the code for the puzzles in the dungeon, including
     * spawning enemies in the dungeon and adding the visual aides for the puzzles.
     * It is annotated accordingly throughout. 
     */
    private void playerInput()
    {
        
        if(Greenfoot.getKey() == "space"){
               attack(currentDirection);
               UI.swing.play();
           }
           
       if ((Greenfoot.isKeyDown("p")==true && UI.dunVal==0)){
                UI.dunVal=7;
                setLocation(getX(), getY());
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
               getWorld().repaint();
               UI.bossWarp=1;
               Spawner.m20.alive=true; getWorld().addObject(Spawner.m20, 180, 100);

        }
           
         if(UI.dunVal==0 && secret==0 && UI.arenaT!=1){ //System.out.println("Entered playerInput()");
           if(Greenfoot.isKeyDown("up") == true && !UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen].getColorAt(getX(), getY()-10).equals(Color.black)){
               currentDirection = Direction.UP;
               changeImage(Direction.UP);
               move(Direction.UP);
               //resetDelay();
           }
            
           else if(Greenfoot.isKeyDown("down") == true && !UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen].getColorAt(getX(), getY()+10).equals(Color.black)){
               currentDirection = Direction.DOWN;
               changeImage(Direction.DOWN);
               move(Direction.DOWN);
               //resetDelay();
           }
           
           else if(Greenfoot.isKeyDown("left") == true && !UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen].getColorAt(getX()-10, getY()).equals(Color.black)){
               currentDirection = Direction.LEFT;
               changeImage(Direction.LEFT);
               move(Direction.LEFT);
               //resetDelay();
           }
           
           else if(Greenfoot.isKeyDown("right") == true && !UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen].getColorAt(getX()+10, getY()).equals(Color.black)){
               currentDirection = Direction.RIGHT;
               changeImage(Direction.RIGHT);
               move(Direction.RIGHT);
               //resetDelay();
               
               
            }
           
                
            else if(Greenfoot.isKeyDown("up") == true && UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen].getColorAt(getX(), getY()-1).equals(Color.green)){
               UI.music.stop();
               UI.dunMusic1.playLoop();
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal], 0, 0);
               getWorld().repaint();
               setLocation(getX()+53, getY()+110);
               UI.enter.play();
               UI.dunVal++;
               Spawner.m20.alive=true; getWorld().addObject(Spawner.m20, 180, 100);
               Spawner.m19.alive=true; getWorld().addObject(Spawner.m19, 100, 100);
               
               
               //resetDelay();
               }
               
               else if(Greenfoot.isKeyDown("up") == true && UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen].getColorAt(getX(), getY()-1).equals(Color.yellow) && UI.arenaT==0){
               
               UI.music.stop();
               UI.dunMusic1.playLoop();
               UI.tempBg.drawImage(UI.arena, 0, 0);
               getWorld().repaint();
               setLocation(getX()+53, getY()+110);
               UI.enter.play();
               UI.arenaT=1;
               if(Spawner.m1.isAlive()==true)Spawner.m1.destroy();
               if(Spawner.m2.isAlive()==true)Spawner.m2.destroy();
               if(Spawner.m3.isAlive()==true)Spawner.m3.destroy();
              arenaSpawner myArena = new arenaSpawner();
               getWorld().addObject(myArena,128,110);
               myArena.getImage().setTransparency(0);
               }
           else{
               resetDelay();
           }//end else if statement
           
        }
        else if(UI.dunVal==0 && secret==1 && UI.arenaT!=1){
            UI.tempBg.drawImage(UI.smask2,0,50);
            UI.smask2.setTransparency(0);
            UI.tempBg.drawImage(UI.s2,0,50);
            getWorld().repaint();
            
            if(Greenfoot.isKeyDown("up") == true && !UI.smask2.getColorAt(getX(), getY()-5).equals(Color.black)){  
               currentDirection = Direction.UP;
               changeImage(Direction.UP);
               move(Direction.UP);
            } 
            else if (Greenfoot.isKeyDown("down") == true && !UI.smask2.getColorAt(getX(), getY()+5).equals(Color.black)){
                
               currentDirection = Direction.DOWN;
               changeImage(Direction.DOWN);
               move(Direction.DOWN); } 
               
            else if (Greenfoot.isKeyDown("left") == true && !UI.smask2.getColorAt(getX()-5, getY()).equals(Color.black)){
                
               currentDirection = Direction.LEFT;
               changeImage(Direction.LEFT);
               move(Direction.LEFT); } 
               
            else if (Greenfoot.isKeyDown("right") == true && !UI.smask2.getColorAt(getX()+5, getY()).equals(Color.black)){
                
               currentDirection = Direction.RIGHT;
               changeImage(Direction.RIGHT);
               move(Direction.RIGHT); } 
               
            else if (Greenfoot.isKeyDown("right") == true && UI.smask2.getColorAt(getX(), getY()).equals(Color.blue)){
               
                   if(fairySecret==0){
                       fairySecret=1;
                   UI.secret.play();
                   getWorld().addObject(LinkWorld.heart2, 201, 34);
                   getWorld().addObject(LinkWorld.heart3, 211, 34);
                   getWorld().addObject(LinkWorld.heart4, 221, 34);
                   getWorld().addObject(LinkWorld.heart5, 231, 34);
                   
                   
                   health=health+4;
                   
                
                }
                   
            } 
                else if (Greenfoot.isKeyDown("up") == true && UI.smask2.getColorAt(getX(), getY()).equals(Color.blue)){
               
                   if(health<9){
                   UI.secret.play();
                   getWorld().addObject(LinkWorld.heart2, 201, 34);
                   getWorld().addObject(LinkWorld.heart3, 211, 34);
                   getWorld().addObject(LinkWorld.heart4, 221, 34);
                   getWorld().addObject(LinkWorld.heart5, 231, 34);
                   health=health+4;}
                   
            }
                else if (Greenfoot.isKeyDown("down") == true && UI.smask2.getColorAt(getX(), getY()).equals(Color.blue)){
               
                   if(health<9){
                   UI.secret.play();
                   getWorld().addObject(LinkWorld.heart2, 201, 34);
                   getWorld().addObject(LinkWorld.heart3, 211, 34);
                   getWorld().addObject(LinkWorld.heart4, 221, 34);
                   getWorld().addObject(LinkWorld.heart5, 231, 34);
                   health=health+4;}
                   
            }
                else if (Greenfoot.isKeyDown("left") == true && UI.smask2.getColorAt(getX(), getY()).equals(Color.blue)){
                    if(health<9){
                   UI.secret.play();
                   getWorld().addObject(LinkWorld.heart2, 201, 34);
                   getWorld().addObject(LinkWorld.heart3, 211, 34);
                   getWorld().addObject(LinkWorld.heart4, 221, 34);
                   getWorld().addObject(LinkWorld.heart5, 231, 34);
                   health=health+4;}
                   
            }
               
            else{
                resetDelay();
            }
        }
        else if(UI.arenaT==1){
            if(Greenfoot.isKeyDown("up") == true && !UI.arena_mask.getColorAt(getX(), getY()-10).equals(Color.black)){
                
               currentDirection = Direction.UP;
               changeImage(Direction.UP);
               move(Direction.UP);
               
            } 
            else if (Greenfoot.isKeyDown("down") == true && !UI.arena_mask.getColorAt(getX(), getY()+10).equals(Color.black)){
                
               currentDirection = Direction.DOWN;
               changeImage(Direction.DOWN);
               move(Direction.DOWN); } 
               
            else if (Greenfoot.isKeyDown("left") == true && !UI.arena_mask.getColorAt(getX()-10, getY()).equals(Color.black)){
                
               currentDirection = Direction.LEFT;
               changeImage(Direction.LEFT);
               move(Direction.LEFT); } 
               
            else if (Greenfoot.isKeyDown("right") == true && !UI.arena_mask.getColorAt(getX()+10, getY()).equals(Color.black)){
                
               currentDirection = Direction.RIGHT;
               changeImage(Direction.RIGHT);
               move(Direction.RIGHT); }
        }
       
        else if(Greenfoot.isKeyDown("up") == true && !UI.dmasks[UI.dunVal-1].getColorAt(getX(), getY()-10).equals(Color.black)){
                
               currentDirection = Direction.UP;
               changeImage(Direction.UP);
               move(Direction.UP);
               
            } 
            else if (Greenfoot.isKeyDown("down") == true && !UI.dmasks[UI.dunVal-1].getColorAt(getX(), getY()+10).equals(Color.black)){
                
               currentDirection = Direction.DOWN;
               changeImage(Direction.DOWN);
               move(Direction.DOWN); } 
               
            else if (Greenfoot.isKeyDown("left") == true && !UI.dmasks[UI.dunVal-1].getColorAt(getX()-10, getY()).equals(Color.black)){
                
               currentDirection = Direction.LEFT;
               changeImage(Direction.LEFT);
               move(Direction.LEFT); } 
               
            else if (Greenfoot.isKeyDown("right") == true && !UI.dmasks[UI.dunVal-1].getColorAt(getX()+10, getY()).equals(Color.black)){
                
               currentDirection = Direction.RIGHT;
               changeImage(Direction.RIGHT);
               move(Direction.RIGHT); } 
               
               /** This case for dungeon level 6 and the switch**/
             else if(Greenfoot.isKeyDown("up") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX(), getY()-1).equals(Color.green) && (UI.dunVal==6)){
                
               if(UI.trigger6==1){ 
               UI.dunVal++;
               setLocation(getX(), getY()+110);
               UI.enter.play();
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
               getWorld().repaint(); 
               if(Spawner.m18.isAlive()==true)Spawner.m18.destroy();
               if(Spawner.m19.isAlive()==true)Spawner.m19.destroy();
               if(Spawner.m20.isAlive()==true)Spawner.m20.destroy();
               
               Spawner.m20.alive=true; getWorld().addObject(Spawner.m20, 180, 100);
            }
               else  { UI.wrong.play(); }
               
                }
               
            else if(Greenfoot.isKeyDown("up") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX(), getY()-1).equals(Color.green) && (UI.dunVal==1 || UI.dunVal==2 ||UI.dunVal==4 )){
               
                if(UI.dunVal==1){ 
                    if(Spawner.m19.isAlive()==true)Spawner.m19.destroy();
                    if(Spawner.m20.isAlive()==true)Spawner.m20.destroy();
                    Spawner.m9.alive=true; getWorld().addObject(Spawner.m9, 180, 160);
                
                    UI.dunVal++;
                    setLocation(getX(), getY()+110);
                    UI.enter.play();
                    UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
                    getWorld().repaint();  
                }
                // 1_2 spawn: m9
                else
                if(UI.dunVal==2){ if(Spawner.m9.isAlive()==true)Spawner.m9.destroy();
                              Spawner.m10.alive=true; getWorld().addObject(Spawner.m10, 180, 160);
                              Spawner.m11.alive=true; getWorld().addObject(Spawner.m11, 70, 100);  
                              
                              UI.dunVal++;
                              setLocation(getX(), getY()+110);
                              UI.enter.play();
                              UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
                              getWorld().repaint();  
                }// 1_3 spawn: m10 and m11
                //SPAWN ENEMIES FOR AREA
                
               else 
               if(UI.dunVal==4) {
                   Enemies.kills=0;
                   UI.miniBoss=0;
               if(Spawner.m10.isAlive()==true)Spawner.m10.destroy();
               if(Spawner.m11.isAlive()==true)Spawner.m11.destroy();
               
               Spawner.m10.alive=true; getWorld().addObject(Spawner.m10, 180, 160);
               Spawner.m11.alive=true; getWorld().addObject(Spawner.m11, 180, 150);   
               Spawner.m12.alive=true; getWorld().addObject(Spawner.m12, 180, 160);
               Spawner.m13.alive=true; getWorld().addObject(Spawner.m13, 180, 150);
               Spawner.m14.alive=true; getWorld().addObject(Spawner.m14, 180, 160);
               Spawner.m15.alive=true; getWorld().addObject(Spawner.m15, 180, 150);
               
               UI.dunVal++; 
               setLocation(getX()+50, getY()+100);
               UI.enter.play();
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
               getWorld().repaint(); } 
               

                
       
            }   
            
            /**Part 1 of the dun9 puzzle**/
            else if(Greenfoot.isKeyDown("up") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX(), getY()-1).equals(Color.MAGENTA) && UI.dunVal==9){
                if(UI.trigger9==0)
                {
                    UI.trigger9++;
                    UI.correct.play();
                }
                if(UI.trigger9==2 || UI.trigger9==3){
                UI.trigger9=0;
                UI.wrong.play();
                
                Spawner.m20.alive=true; getWorld().addObject(Spawner.m20, 100, 100);
               Spawner.m19.alive=true; getWorld().addObject(Spawner.m19, 100, 150);
               Spawner.m18.alive=true; getWorld().addObject(Spawner.m18, 180, 100);
               Spawner.m11.alive=true; getWorld().addObject(Spawner.m11, 110, 180);
                }
                
            }
            
            /**Part 2 of the dun9 puzzle**/
            else if(Greenfoot.isKeyDown("left") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX()-1, getY()).equals(Color.YELLOW) && UI.dunVal==9){
                if(UI.trigger9==1)
                {
                    UI.trigger9++;
                    UI.correct.play();
                }
                if(UI.trigger9==0 || UI.trigger9==3){
                UI.trigger9=0;
                UI.wrong.play();
                
                Spawner.m20.alive=true; getWorld().addObject(Spawner.m20, 100, 100);
               Spawner.m19.alive=true; getWorld().addObject(Spawner.m19, 100, 150);
               Spawner.m18.alive=true; getWorld().addObject(Spawner.m18, 180, 100);
               Spawner.m11.alive=true; getWorld().addObject(Spawner.m11, 110, 180);
                }
                
            }
            
            /**Part 3 of the dun9 puzzle**/
            else if(Greenfoot.isKeyDown("left") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX()-1, getY()).equals(Color.red) && UI.dunVal==9){
                if(UI.trigger9==2)
                {
                    UI.trigger9++;
                    UI.correct.play();
                }
                if(UI.trigger9==0 || UI.trigger9==1){
                UI.trigger9=0;
                UI.wrong.play();
                
               Spawner.m20.alive=true; getWorld().addObject(Spawner.m20, 100, 100);
               Spawner.m19.alive=true; getWorld().addObject(Spawner.m19, 100, 150);
               Spawner.m18.alive=true; getWorld().addObject(Spawner.m18, 180, 100);
               Spawner.m11.alive=true; getWorld().addObject(Spawner.m11, 110, 180);
                }
            }
            
            /**Part 4 of the dun9 puzzle**/
            else if(Greenfoot.isKeyDown("right") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX()-1, getY()).equals(Color.green) && UI.dunVal==9){
                if(UI.trigger9==3)
                {
                    UI.trigger9++;
                    UI.secret.play();
                    
                    if(UI.trigger9==4 && UI.trigger8==1)
                    {
                        UI.bossWarp=1;
                        UI.warp.play();
                    }
                }
                if(UI.trigger9==0 || UI.trigger9==1 || UI.trigger9==2){
                UI.trigger9=0;
                UI.wrong.play();
                
               Spawner.m20.alive=true; getWorld().addObject(Spawner.m20, 100, 100);
               Spawner.m19.alive=true; getWorld().addObject(Spawner.m19, 100, 150);
               Spawner.m18.alive=true; getWorld().addObject(Spawner.m18, 180, 100);
               Spawner.m11.alive=true; getWorld().addObject(Spawner.m11, 110, 180);
                
                }
                
            }
            
            /** The warp activation of dun7 to reach the final room and the boss**/
            else if((Greenfoot.isKeyDown("up") == true || Greenfoot.isKeyDown("left") == true || Greenfoot.isKeyDown("right") == true || Greenfoot.isKeyDown("down") == true )&& UI.dmasks[UI.dunVal-1].getColorAt(getX(), getY()).equals(Color.red) && UI.dunVal==7){
                if(UI.bossWarp==1 ){
                UI.warp.play();
                UI.dunVal=10;
                setLocation(getX(), getY());
                UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
                getWorld().repaint();
                
                if(Spawner.m20.isAlive()==true)Spawner.m20.destroy();
                
               
            }
            else{
                UI.wrong.play();}
            }
            
            /** Secret room 1 entry**/
            else if(Greenfoot.isKeyDown("left") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX()-1, getY()).equals(Color.red) && UI.dunVal==2){
               UI.dunVal=11;
               setLocation(getX()+185, getY()+20);
               UI.enter.play();
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
               getWorld().repaint();
               
               if(Spawner.m9.isAlive()==true)Spawner.m9.destroy();
               
               
            }
            
            /** Not used yet**/
            else if (Greenfoot.isKeyDown("down") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX(), getY()+1).equals(Color.green) && UI.dunVal==-1){
                
               UI.dunVal++; 
               setLocation(getX(), getY()-110);
               UI.enter.play();
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
               getWorld().repaint(); } 
               
            else if (Greenfoot.isKeyDown("left") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX()-1, getY()).equals(Color.green) && (UI.dunVal==3 || UI.dunVal==7)){
                if(UI.dunVal==3){
                if(Spawner.m10.isAlive()==true)Spawner.m10.destroy();
                if(Spawner.m11.isAlive()==true)Spawner.m11.destroy();
                
                Spawner.m10.alive=true; getWorld().addObject(Spawner.m10, 43, 160);
                Spawner.m11.alive=true; getWorld().addObject(Spawner.m11, 70, 180);  
            }
                if(UI.dunVal==7){
                   if(UI.trigger8==0){

                       Key.key8.alive=true; getWorld().addObject(Key.key8, 43, 100);
                       Key.key8.right=0; Key.key8.left=0; Key.key8.up=0; Key.key8.down=0; Key.key8.direction=0;
                       
                    }
                    if(Spawner.m20.isAlive()==true)Spawner.m20.destroy();
                }
                
               UI.dunVal++;
               setLocation(getX()+185, getY());
               UI.enter.play();
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
               getWorld().repaint();}
               
            else if (Greenfoot.isKeyDown("right") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX()+1, getY()).equals(Color.green)&& UI.dunVal==7){
               if(Spawner.m20.isAlive()==true)Spawner.m20.destroy();
               
               
               UI.dunVal=9;
               setLocation(getX()-185, getY());
               UI.enter.play();
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
               getWorld().repaint(); 
            }   
            else if (Greenfoot.isKeyDown("right") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX()+1, getY()).equals(Color.blue)){
                
               if(UI.dunVal==4){
               if(Spawner.m10.isAlive()==true)Spawner.m10.destroy();
               if(Spawner.m11.isAlive()==true)Spawner.m11.destroy();
               Spawner.m10.alive=true; getWorld().addObject(Spawner.m10, 180, 160);
               Spawner.m11.alive=true; getWorld().addObject(Spawner.m11, 110, 150);  
                   
               UI.dunVal--;
               setLocation(getX()-185, getY());
               UI.enter.play();
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
               getWorld().repaint(); }
               
  
                 if(UI.dunVal==6 && UI.trigger6==0){
                    UI.secret.play();
                    UI.trigger6=1;
                
                }
                if(UI.dunVal==8){
               UI.dunVal--;
               setLocation(getX()-185, getY());
               UI.enter.play();
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
               getWorld().repaint(); 
               Spawner.m20.alive=true; getWorld().addObject(Spawner.m20, 100, 100);
               
               if(UI.trigger8==0){Key.key8.removeSelf();}
               if(UI.bossWarp==1){
                   UI.enter.play();
                   
                }
                
               
                }
                if(UI.dunVal==11){
                    UI.dunVal=2;
                    setLocation(getX()-185, getY());
                    UI.enter.play();
                    UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
                    getWorld().repaint(); 
                    
                   
                   
                    Spawner.m9.alive=true; getWorld().addObject(Spawner.m9, 180, 160);
                }
                
            }
            else if (Greenfoot.isKeyDown("left") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX()-1, getY()).equals(Color.blue)){
               if(UI.dunVal==9){
               UI.dunVal=7;
               setLocation(getX()+185, getY());
               UI.enter.play();
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
               getWorld().repaint(); 
            
              if(Spawner.m19.isAlive()==true)Spawner.m19.destroy();
              if(Spawner.m18.isAlive()==true)Spawner.m18.destroy();
              if(Spawner.m20.isAlive()==true)Spawner.m20.destroy();
              if(Spawner.m11.isAlive()==true)Spawner.m11.destroy();
               
               Spawner.m20.alive=true; getWorld().addObject(Spawner.m20, 100, 100);
               
            }
            } 
            else if (Greenfoot.isKeyDown("up") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX()+1, getY()).equals(Color.blue)){
                
               if(UI.dunVal==-1){
               UI.dunVal--;
               setLocation(getX(), getY()+110);
               UI.enter.play();
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
               getWorld().repaint(); }
            }
            else if (Greenfoot.isKeyDown("down") == true && UI.dmasks[UI.dunVal-1].getColorAt(getX()+1, getY()).equals(Color.blue)){
               
                if(UI.dunVal==7){
                UI.dunVal--;
                setLocation(getX(), getY()-110);
                UI.enter.play();
                UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
                getWorld().repaint();
                if(Spawner.m20.isAlive()==true)Spawner.m20.destroy();
                Spawner.m20.alive=true; getWorld().addObject(Spawner.m20, 100, 100);
                Spawner.m19.alive=true; getWorld().addObject(Spawner.m19, 100, 150);
                Spawner.m18.alive=true; getWorld().addObject(Spawner.m18, 180, 160);
                
                
                }
                if(UI.dunVal==2){
                if(Spawner.m9.isAlive()==true)Spawner.m9.destroy();
                Spawner.m19.alive=true; getWorld().addObject(Spawner.m19, 100, 160);
                Spawner.m20.alive=true; getWorld().addObject(Spawner.m20, 200, 160);
                }
                if(UI.dunVal==3){
                if(Spawner.m10.isAlive()==true)Spawner.m10.destroy();
                if(Spawner.m11.isAlive()==true)Spawner.m11.destroy();
                Spawner.m9.alive=true; getWorld().addObject(Spawner.m9, 180, 160);
                }
                if(UI.dunVal==1){
                if(Spawner.m19.isAlive()==true)Spawner.m19.destroy();
                if(Spawner.m20.isAlive()==true)Spawner.m20.destroy();   
                UI.dunVal=0;
                UI.dunMusic1.stop();
                UI.music.playLoop();
                setLocation(getX()-53, getY()-110);
                UI.enter.play();
                UI.tempBg.drawImage(UI.visualMap[0][0], 0, 50);
                UI.horizontalScreen=0;
                UI.verticalScreen=0;
                getWorld().repaint(); 
                }else if(UI.dunVal==2 || UI.dunVal==3){
               UI.dunVal--;
               setLocation(getX(), getY()-110);
               UI.enter.play();
               UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
               getWorld().repaint(); }
            } 
             else if ((Greenfoot.isKeyDown("up") == true || Greenfoot.isKeyDown("left") == true || Greenfoot.isKeyDown("right") == true )&& UI.dmasks[UI.dunVal-1].getColorAt(getX()+1, getY()).equals(Color.red) && UI.dunVal==3){
                 setLocation(137, 190);
                 UI.wrong.play();
            }
            else if (((Greenfoot.isKeyDown("up") == true || Greenfoot.isKeyDown("right") == true)&& UI.dmasks[UI.dunVal-1].getColorAt(getX(), getY()-1).equals(Color.red))){
                 
                if(UI.dunVal==5 && UI.miniBoss==1)
                {
                   UI.dunMusic1.stop();
                   UI.dunMusic2.playLoop();
                   UI.dunVal++; 
                   UI.tempBg.drawImage(UI.dlevel[UI.dunVal-1], 0, 0);
                   setLocation(getX(), getY()+100);
                   UI.warp.play();
                   getWorld().repaint();
                   
                   Spawner.m20.alive=true; getWorld().addObject(Spawner.m20, 100, 100);
                   Spawner.m19.alive=true; getWorld().addObject(Spawner.m19, 100, 150);
                   Spawner.m18.alive=true; getWorld().addObject(Spawner.m18, 180, 100);
                }
                

               
                
                // setLocation(137, 190);
            }
            else{
               resetDelay();
            }//end else if statement 
        
    }//end playerInput
    
    /**
     * Whenever Link reaches the edge of a screen, this function will be called
     * to 1) shift the screen over into a new screen, 2) change Link's position 
     * to the new screen, and 3) update the colormasking array in UI. Also, all
     * enemies are deleted once entering a new screen, and a new batch of enemies
     * are spawned. Screen scrolling is done via interpolation, and Link travels
     * to the next screen the same way.
     */
    
    public void changeScreenRight()
    {
        GreenfootImage currentScreen = UI.visualMap[UI.horizontalScreen][UI.verticalScreen]; 
        GreenfootImage nextScreen = UI.visualMap[UI.horizontalScreen+1][UI.verticalScreen];
        int scrWidth = currentScreen.getWidth();
        double scrollTemp = 0.0;
        double oldX = getX();
        double margin = getX();
        double dist = Math.abs(scrWidth-(margin*2))+2;
        
        for(int i = 0; i < shift; i++)
        {
            Greenfoot.delay(2);
            scrollTemp = ((double)(i+1)/(double)shift)*(getWorld().getWidth());
            newScreenPos = scrWidth-(int)(Math.round(scrollTemp));
            oldScreenPos = -(int)(Math.round(scrollTemp));
            UI.tempBg.drawImage(currentScreen, oldScreenPos, 50);            
            UI.tempBg.drawImage(nextScreen, newScreenPos, 50);
            int nextX = (int)Math.round(oldX - dist*((double)(i)/(double)shift));
            setLocation(nextX, getY());
        }
        
        UI.horizontalScreen++;
        getWorld().repaint();}
        
    /**
     * When Link reaches the left side of the screen and can access the next screen
     * , the screen will scroll left.
     */
    public void changeScreenLeft()
    {
        GreenfootImage currentScreen = UI.visualMap[UI.horizontalScreen][UI.verticalScreen]; 
        GreenfootImage nextScreen = UI.visualMap[UI.horizontalScreen-1][UI.verticalScreen];
        int scrWidth = currentScreen.getWidth();
        double scrollTemp = 0.0;
        double oldX = getX();
        double margin = getX();
        double dist = Math.abs(scrWidth-(margin*2))+2;
        
        for(int i = 0; i < shift; i++)
        {
            Greenfoot.delay(2);
            scrollTemp = ((double)(i+1)/(double)shift)*(getWorld().getWidth());
            newScreenPos = -scrWidth+(int)(Math.round(scrollTemp));
            oldScreenPos = (int)(Math.round(scrollTemp));
            UI.tempBg.drawImage(currentScreen, oldScreenPos, 50);            
            UI.tempBg.drawImage(nextScreen, newScreenPos, 50);
            int nextX = (int)Math.round(oldX + dist*((double)(i)/(double)shift));
            setLocation(nextX, getY());
        }
        UI.horizontalScreen--;
        getWorld().repaint();
    }    
    
    /**
     * When Link reaches the left side of the screen and can access the next screen
     * , the screen will scroll up.
     */
    public void changeScreenUp()
    {
        GreenfootImage currentScreen = UI.visualMap[UI.horizontalScreen][UI.verticalScreen]; 
        GreenfootImage nextScreen = UI.visualMap[UI.horizontalScreen][UI.verticalScreen+1];         
        int scrHeight = currentScreen.getHeight();
        double scrollTemp = 0.0;
        double oldY = getY();
        double margin = getY()-50;
        double dist = Math.abs(scrHeight-(margin*2))+2;
        
        for(int i = 0; i < shift; i++)
        {
            Greenfoot.delay(2);
            scrollTemp = ((double)(i+1)/(double)shift)*(scrHeight);
            newScreenPos = -scrHeight+(int)(Math.round(scrollTemp));
            oldScreenPos = (int)(Math.round(scrollTemp));
            UI.tempBg.drawImage(nextScreen, 0, newScreenPos + 50);
            UI.tempBg.drawImage(currentScreen, 0, oldScreenPos + 50);
            UI.tempBg.drawImage(UI.bar, 0, 0);
            int nextY = (int)Math.round(oldY + dist*((double)(i)/(double)shift) );
            setLocation(getX(), nextY);
        }
        UI.verticalScreen++;
        getWorld().repaint();
    }     
    
    /**
     * When Link reaches the left side of the screen and can access the next screen
     * , the screen will scroll down.
     */
    public void changeScreenDown()
    {
        GreenfootImage currentScreen = UI.visualMap[UI.horizontalScreen][UI.verticalScreen]; 
        GreenfootImage nextScreen = UI.visualMap[UI.horizontalScreen][UI.verticalScreen-1];         
        int scrHeight = currentScreen.getHeight();
        double scrollTemp = 0.0;
        double oldY = getY();
        double margin = getWorld().getHeight()-getY();
        double dist = Math.abs(scrHeight-(margin*2))+2;
        
        for(int i = 0; i < shift; i++)
        {
            Greenfoot.delay(2);
            scrollTemp = ((double)(i+1)/(double)shift)*(scrHeight);
            newScreenPos = scrHeight-(int)(Math.round(scrollTemp));
            oldScreenPos = -(int)(Math.round(scrollTemp));
            UI.tempBg.drawImage(nextScreen, 0, newScreenPos + 50);
            UI.tempBg.drawImage(currentScreen, 0, oldScreenPos + 50);
            UI.tempBg.drawImage(UI.bar, 0, 0);
            int nextY = (int)Math.round(oldY - dist*((double)(i)/(double)shift) );
            setLocation(getX(), nextY);
        }
        UI.verticalScreen--;
        getWorld().repaint();
    }   
    
    /**
     * Moves link in the direction that corresponds with the currently pressed arrow key
     */
    public void move(Direction dir, int moveSpeed)
    {
        int i=0;
        switch(dir){
            case UP:
                if(UI.dunVal > 0 && health>1) {
                    i = moveSpeed;
                    while(UI.dmasks[UI.dunVal-1].getColorAt(getX(), getY()-i).equals(Color.black)) {
                        i-=2;
                    }
                } else if (UI.dunVal == 0 && health>1) {
                    i = moveSpeed;
                    while(UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen].getColorAt(getX(), getY()-i).equals(Color.black)) {
                        i-=2;
                    }
                }
                else if (UI.arenaT==1 && health>1){
                    i = 6;
                    while(UI.arena_mask.getColorAt(getX(), getY()+i).equals(Color.black)) {
                        i-=2;
                    }
                }
                
                setLocation(getX(), getY() - i);
                break;
            case DOWN:
                if(UI.dunVal > 0 && health>1) {
                    i = 6;
                    while(UI.dmasks[UI.dunVal-1].getColorAt(getX(), getY()+i).equals(Color.black)) {
                        i-=2;
                    }
                } else if (UI.dunVal == 0 && health>1) {
                    i = moveSpeed;
                    while(UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen].getColorAt(getX(), getY()+i).equals(Color.black)) {
                        i-=2;
                    }
                }
                else if (UI.arenaT==1 && health>1){
                    i = 6;
                    while(UI.arena_mask.getColorAt(getX(), getY()+i).equals(Color.black)) {
                        i-=2;
                    }
                }
                setLocation(getX(), getY() + i);
                break;
            case RIGHT:
                if(UI.dunVal > 0 && health>1) {
                    i = 6;
                    while(UI.dmasks[UI.dunVal-1].getColorAt(getX()+i, getY()).equals(Color.black)) {
                        i-=2;
                    }
                } else if (UI.dunVal == 0 && health>1) {
                    i = moveSpeed;
                    while(UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen].getColorAt(getX()+i, getY()).equals(Color.black)) {
                        i-=2;
                    }
                }
                else if (UI.arenaT==1 && health>1){
                    i = 6;
                    while(UI.arena_mask.getColorAt(getX(), getY()+i).equals(Color.black)) {
                        i-=2;
                    }
                }
                setLocation(getX() + i, getY());
                break;
            case LEFT:
                if(UI.dunVal > 0 && health>1) {
                    i = 6;
                    while(UI.dmasks[UI.dunVal-1].getColorAt(getX()-i, getY()).equals(Color.black)) {
                        i-=2;
                    }
                } else if (UI.dunVal == 0 && health>1) {
                    i = moveSpeed;
                    while(UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen].getColorAt(getX()-i, getY()).equals(Color.black)) {
                        i-=2;
                    }
                }
                else if (UI.arenaT==1 && health>1){
                    i = 6;
                    while(UI.arena_mask.getColorAt(getX(), getY()+i).equals(Color.black)) {
                        i-=2;
                    }
                }
                setLocation(getX() - i, getY());
                break;
                
        }//end switch
        
    }//end move

    /**
     * Moves the screen in the direction of the param
     * @param dir
     *      Is the direction the screen needs to move
     */
    public void move(Direction dir)
    {
        switch(dir){
            case UP:
                if(getY()-5 <= 60 && secret==0)
                {
                    getWorld().removeObjects(getWorld().getObjects(Moblin.class));
                    changeScreenUp();
                    //spawnEnemies(UI.horizontalScreen, UI.verticalScreen);
                    break;
                }
                else
                {
                    setLocation(getX(), getY() - moveSpeed);
                    break;                    
                }            
            case DOWN:
                if(getY()+5 >= getWorld().getHeight()-10 && secret==0)
                {
                    getWorld().removeObjects(getWorld().getObjects(Moblin.class));
                    changeScreenDown();
                    //spawn.spawnEnemies(UI.horizontalScreen, UI.verticalScreen);
                    break;
                }
                else
                {
                    setLocation(getX(), getY() + moveSpeed);
                    break;                    
                }   
            case RIGHT:
                if(getX()+5 >= getWorld().getWidth()-10 && secret==0)
                {
                    if(UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen].getColorAt(getX(),getY()).equals(Color.red)){
                         setLocation(55,100);
                         secret=1;
                         if(Spawner.m6.isAlive()==true) Spawner.m6.destroy();
                         if(Spawner.m5.isAlive()==true) Spawner.m5.destroy();
                         if(Spawner.m4.isAlive()==true) Spawner.m4.destroy();
                     }
                    else{
                        getWorld().removeObjects(getWorld().getObjects(Moblin.class));
                        changeScreenRight();
                    }
                    //spawn.spawnEnemies(UI.horizontalScreen, UI.verticalScreen);
                    break;
                }
                else
                {
                    setLocation(getX() + moveSpeed, getY());
                    break;                    
                }
            case LEFT:
                if(getX()-5 <= 10)
                {
                    if(UI.smask2.getColorAt(getX(), getY()).equals(Color.red)){
                        UI.tempBg.drawImage(UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen],0,50);
                        UI.colorMaskMap[UI.horizontalScreen][UI.verticalScreen].setTransparency(0);
                        UI.tempBg.drawImage(UI.visualMap[UI.horizontalScreen][UI.verticalScreen],0,50);
                        getWorld().repaint();
                        setLocation(getX()+175,getY());
                        secret=0;
                        Spawner.m6.alive=true; getWorld().addObject(Spawner.m6, 100, 100);
                        Spawner.m5.alive=true; getWorld().addObject(Spawner.m5, 100, 150);
                        Spawner.m4.alive=true; getWorld().addObject(Spawner.m4, 150, 100);
                    }   
                    else{
                        getWorld().removeObjects(getWorld().getObjects(Moblin.class));
                        changeScreenLeft();
                    }
                    //spawn.spawnEnemies(UI.horizontalScreen, UI.verticalScreen);
                    break;
                }
                else
                {
                    setLocation(getX() - moveSpeed, getY());
                    break;                    
                }
                
        }//end switch
        
    }//end move
    
    
    
    /**
     * Changes images of Link based on direction he is moving
     */
    public void changeImage(Direction dir)
    {   
        //If the image is currently 0 (e.g. "linkUp0.png"), and the delay is 50, 
        //change image to the same direction but next incrementation.
        if(isImage1 == false && delay == FINAL_DELAY){
            switch(dir){
                case UP:
                    setImage("linkUp1.png");
                    break;
                case DOWN:
                    setImage("linkDown1.png");
                    break;
                case RIGHT:
                    setImage("linkRight1.png");
                    break;
                case LEFT:
                    setImage("linkLeft1.png");
                    break;                
            }//end switch
            isImage1 = true;
            delay = 0;
        }
        
        else if(isImage1 == true && delay == FINAL_DELAY){
            switch(dir){
                case UP:
                    setImage("linkUp0.png");
                    break;
                case DOWN:
                    setImage("linkDown0.png");
                    break;
                case RIGHT:
                    setImage("linkRight0.png");
                    break;
                case LEFT:
                    setImage("linkLeft0.png");
                    break;                
            }//end switch
            isImage1 = false;
            delay = 0;
        }
        
        else{
            //Increments delay, giving a delay of 50 in between every image change
            delay = delay + 1;
        }
    }//end changeImage()
    
    /**
     * Resets delay to specified value for smoother animations
     */
    private static void resetDelay()
    {
        delay = FINAL_DELAY;
    }//end resetDelay

    /**
     * Sets the image for Link when he attacks
     */ 
    private void attack(Direction currentDirection)
    {
        //System.out.println("Entered attack()");
        timer = 0;
        switch(currentDirection){
                case UP:
                    setImage(LinkSwordUp);
                    isAttacking = true;
                    attackDirection = Direction.UP;
                    //isHitting();
                    break;
                case DOWN:
                    setImage(LinkSwordDown);
                    isAttacking = true;
                    attackDirection = Direction.DOWN;
                    //isHitting();
                    break;
                case RIGHT:
                    setImage(LinkSwordRight);
                    isAttacking = true;
                    attackDirection = Direction.RIGHT;
                    //isHitting();
                    break;
                case LEFT:
                    setImage(LinkSwordLeft);
                    isAttacking = true;
                    attackDirection = Direction.LEFT;
                    //isHitting();
                    break;                
            }//end switch
    }//end attack()
    
    /**
     * Resets the images of Link when he is done attacking.
     * @param attackDir
     *      The direction Link was facing before method as called.
     */
    private void stopAttacking(Direction attackDir){
        switch(attackDir){
                case UP:
                    setImage("linkUp0.png");
                    isAttacking = false;
                    break;
                case DOWN:
                    setImage("linkDown0.png");
                    isAttacking = false;
                    break;
                case RIGHT:
                    setImage("linkRight0.png");
                    isAttacking = false;
                    break;
                case LEFT:
                    setImage("linkLeft0.png");
                    isAttacking = false;
                    break;                
            }//end switch
    }
    
    /**
     * Custom delay method. Never fully implemented.
     */
    private void customDelay()
    {
        //Use modulus of a large number
        //setImage(LinkSwordDown);
        
        while(timer % 9000000 != 0){
            //System.out.println(timer);            
            timer++;
        }
        
        timer++;
            
    }//end customDelay()
    
    /**
     * Returns true if Link is hitting.
     */
    private boolean isHitting()
    {
        return true;
    }
    
    /**
     * Returns true if Link is attacking
     */
    public boolean isAttacking()
    {
        if(getImage() == LinkSwordDown || getImage() == LinkSwordUp || getImage() == LinkSwordLeft || getImage() == LinkSwordRight){
            return true;
        }else{
            return false;
        }//end if/else statement
    }//end isAttacking()
    
    /**
     * Decreases this enemy's health equal to amount.
     * 
     * @param amount The amount of health subtracted from this enemy
     */
    public static void decreaseHealth(int amount)
    {
        
        health = health - amount; UI.hurt.play();
        if(health <= 0){}
            //Greenfoot.stop();
    }
    
    /**
     * Decreases Link's health when hit, and causes knockback recovery to 
     * start.
     */
    public static void linkIsHit()
    {        
        if(recovery==0){
            recovery=16;
        LinkWorld.link.knockback(currentDirection);
        if(health == 5){
            LinkWorld.heart5.removeSelf();
        }else if(health == 4){
            LinkWorld.heart4.removeSelf();
        }else if(health == 3){
            LinkWorld.heart3.removeSelf();
        }else if(health == 2){
            LinkWorld.heart2.removeSelf();
        }else if(health == 1){
            LinkWorld.heart1.removeSelf();
            isGameOver = true;
            //System.out.println("gameover");
        }
        decreaseHealth(1);}
    }
    
    
    /**
     * When Link is hit, this method is called. Causes knockback in the direction
     * of the param.
     * @param currentDirection
     *      The direction Link is facing
     */
    public void knockback(Direction currentDirection)
    {
        //int counter = 0;
        isKnockback = true;
        
        switch(currentDirection){
                case UP:
                    if(Greenfoot.isKeyDown("up")){
                        //isKnockback = true;
                        knockbackDirection = Direction.DOWN;
                    }else{
                        if(Moblin.currentDir == 1){
                            //move(Direction.DOWN, distance);
                            knockbackDirection = Direction.DOWN;
                        }else if(Moblin.currentDir == 2){
                            //move(Direction.LEFT, distance);
                            knockbackDirection = Direction.LEFT;
                        }else if(Moblin.currentDir == 3){
                            //move(Direction.UP, distance);
                            knockbackDirection = Direction.UP;
                        }else if(Moblin.currentDir == 0){
                            //move(Direction.RIGHT, distance);
                            knockbackDirection = Direction.RIGHT;
                        }
                    }                    
                    break;
                case DOWN:
                    if(Greenfoot.isKeyDown("down")){
                        //isKnockback = true;
                        knockbackDirection = Direction.UP;
                    }else{
                        if(Moblin.currentDir == 1){
                            //move(Direction.DOWN, distance);
                            knockbackDirection = Direction.DOWN;
                        }else if(Moblin.currentDir == 2){
                            //move(Direction.LEFT, distance);
                            knockbackDirection = Direction.LEFT;
                        }else if(Moblin.currentDir == 3){
                            //move(Direction.UP, distance);
                            knockbackDirection = Direction.UP;
                        }else if(Moblin.currentDir == 0){
                            //move(Direction.RIGHT, distance);
                            knockbackDirection = Direction.RIGHT;
                        }
                    }   
                    break;
                case RIGHT:
                    if(Greenfoot.isKeyDown("right")){
                        //isKnockback = true;
                        knockbackDirection = Direction.LEFT;
                    }else{
                        if(Moblin.currentDir == 1){
                            //move(Direction.DOWN, distance);
                            knockbackDirection = Direction.DOWN;
                        }else if(Moblin.currentDir == 2){
                            //move(Direction.LEFT, distance);
                            knockbackDirection = Direction.LEFT;
                        }else if(Moblin.currentDir == 3){
                            //move(Direction.UP, distance);
                            knockbackDirection = Direction.UP;
                        }else if(Moblin.currentDir == 0){
                            //move(Direction.RIGHT, distance);
                            knockbackDirection = Direction.RIGHT;
                        }
                    }   
                    break;
                case LEFT:
                    if(Greenfoot.isKeyDown("left")){
                        //isKnockback = true;
                        knockbackDirection = Direction.RIGHT;
                    }else{
                        if(Moblin.currentDir == 1){
                            //move(Direction.DOWN, distance);
                            knockbackDirection = Direction.DOWN;
                        }else if(Moblin.currentDir == 2){
                            //move(Direction.LEFT, distance);
                            knockbackDirection = Direction.LEFT;
                        }else if(Moblin.currentDir == 3){
                            //move(Direction.UP, distance);
                            knockbackDirection = Direction.UP;
                        }else if(Moblin.currentDir == 0){
                            //move(Direction.RIGHT, distance);
                            knockbackDirection = Direction.RIGHT;
                        }
                    }   
                    break;                
        }//end switch
    }
    
}